@@ -4,7 +4,8 @@
 # Pod stripped from pm file by OODoc 2.01.
  
 package XML::Compile::Translate::Writer;
-our $VERSION = '1.43';
+use vars '$VERSION';
+$VERSION = '1.45';
 
 use base 'XML::Compile::Translate';
 
@@ -14,7 +15,7 @@ no warnings 'once', 'recursion';
 
 use Log::Report   qw/xml-compile/;
 use List::Util    qw/first/;
-use Scalar::Util  qw/blessed/;
+use Scalar::Util  qw/blessed weaken/;
 use XML::Compile::Util qw/pack_type unpack_type type_of_node SCHEMA2001i
   odd_elements even_elements/;
 
@@ -197,10 +198,13 @@ sub makeChoice($@)
         my ($doc, $values) = @_;
         defined $values or return ();
         foreach my $take (keys %do)
-        {   return $do{$take}->($doc, delete $values->{$take})
+        {
+#warn "TAKE($take) = ", (defined $values->{$take} ? 'defined' : "undef");
+            return $do{$take}->($doc, delete $values->{$take})
                 if defined $values->{$take};
         }
 
+#warn "TRY SPECIALS";
         my $starter = keys %$values;
         foreach (@specials)
         {   my @d = try { $_->($doc, $values) };
@@ -216,6 +220,7 @@ sub makeChoice($@)
             return @d;
         }
 
+#warn "BLURK!";
         # blurk... any element with minOccurs=0 or default?
         foreach (values %do)
         {   my @d = try { $_->($doc, undef) };
@@ -477,6 +482,7 @@ sub nil($)
 
 sub makeComplexElement
 {   my ($self, $path, $tag, $elems, $attrs, $any_attr,undef, $is_nillable) = @_;
+my @e = @$elems;
     my @elems   = odd_elements @$elems;
     my @attrs   = @$attrs;
     my $tags    = join ', ', grep defined
@@ -498,15 +504,15 @@ sub makeComplexElement
                 or error __x"complex `{tag}' requires data at {path}"
                       , tag => $tag, path => $path, _class => 'misfit';
 
-            error __x"complex `{tag}' requires a HASH of input data, not `{found}' at {path}"
-               , tag => $tag, found => (ref $data || $data), path => $path;
+            error __x"complex `{tag}' requires a HASH of input data, not `{got}' at {path}"
+               , tag => $tag, got => (ref $data || $data), path => $path;
         }
 
         my $copy   = { %$data };  # do not destroy callers hash
 
         my @childs = ($is_nillable && (delete $copy->{_} || '') eq 'NIL')
           ? $doc->createAttribute($nilattr => 'true')
-          : map {$_->($doc, $copy)} @elems;
+          : map($_->($doc, $copy), @elems);
 
         for(my $i=0; $i<@attrs; $i+=2)
         {   push @childs, $attrs[$i+1]->($doc, delete $copy->{$attrs[$i]});
@@ -517,7 +523,7 @@ sub makeComplexElement
 
         if(%$copy)
         {   my @not_used
-              = defined $iut ? grep({$_ !~ $iut} keys %$copy) : keys %$copy;
+              = defined $iut ? (grep $_ !~ $iut, keys %$copy) : keys %$copy;
 
             if(@not_used)
             {   trace "available tags are: $tags";
@@ -627,6 +633,7 @@ sub makeMixedElement
             };
     }
 
+    my $iut     = $self->{ignore_unused_tags};
     sub { my ($doc, $data) = @_;
           defined $data or return;
 
@@ -654,10 +661,15 @@ sub makeMixedElement
           push @childs, $_->($doc, $copy)
               for @anya;
 
-          if(my @not_used = sort keys %$copy)
-          {   error __xn "tag `{tags}' not processed at {path}"
-                       , "unprocessed tags {tags} at {path}"
-                       , scalar @not_used, tags => \@not_used, path => $path;
+          if(%$copy)
+          {   my @not_used
+                = defined $iut ? (grep $_ !~ $iut, keys %$copy) : keys %$copy;
+
+              if(my @not_used = sort keys %$copy)
+              {   error __xn "tag `{tags}' not processed at {path}"
+                    , "unprocessed tags {tags} at {path}", scalar @not_used
+                    , tags => [sort @not_used], path => $path;
+              }
           }
 
           @childs or return $node;
@@ -803,7 +815,7 @@ sub makeAttributeRequired
 {   my ($self, $path, $ns, $tag, $label, $do) = @_;
 
     sub { my $value = $do->(@_);
-          return $_[0]->createAttributeNS($ns, $tag, $value)
+          return $_[0]->createAttribute($tag, $value)
               if defined $value;
 
           error __x"attribute `{tag}' is required at {path}"
@@ -874,8 +886,8 @@ sub makeAttributeFixed
 
 # any
 
-sub _splitAnyList($$$)
-{   my ($self, $path, $type, $v) = @_;
+sub _split_any_list($$$)
+{   my ($path, $type, $v) = @_;
     my @nodes = ref $v eq 'ARRAY' ? @$v : defined $v ? $v : return ([], []);
     my (@attrs, @elems);
 
@@ -903,8 +915,11 @@ sub _splitAnyList($$$)
 
 sub makeAnyAttribute
 {   my ($self, $path, $handler, $yes, $no, $process) = @_;
-    my %yes = map +($_ => 1), @{$yes || []};
-    my %no  = map +($_ => 1), @{$no  || []};
+    my %yes   = map +($_ => 1), @{$yes || []};
+    my %no    = map +($_ => 1), @{$no  || []};
+    my $prefs = $self->{prefixes};
+
+    weaken $self;
 
     bless
     sub { my ($doc, $values) = @_;
@@ -918,8 +933,9 @@ sub makeAnyAttribute
               }
               elsif(index($label, ':') >= 0)
               {   (my $prefix, $local) = split ':', $label, 2;
-                  $ns    = $self->namespaceForPrefix($prefix);
-                  $type  = pack_type $ns, $local;
+                  my $match = first {$_->{prefix} eq $prefix} values %$prefs;
+                  my $ns    = $match ? $match->{uri} : undef;
+                  $type     = pack_type $ns, $local;
               }
               else {next}  # not fully qualified, not an 'any'
 
@@ -927,7 +943,7 @@ sub makeAnyAttribute
               $no{$ns} and next if keys %no;
 
               my $value = delete $values->{$label} or next;
-              my ($attrs, $elems) = $self->_splitAnyList($path, $type, $value);
+              my ($attrs, $elems) = _split_any_list $path, $type, $value;
 
               $values->{$type} = $elems if @$elems;
               @$attrs or next;
@@ -948,11 +964,13 @@ sub makeAnyAttribute
 
 sub makeAnyElement
 {   my ($self, $path, $handler, $yes, $no, $process, $min, $max) = @_;
-    my %yes = map +($_ => 1), @{$yes || []};
-    my %no  = map +($_ => 1), @{$no  || []};
-    my $prefixes = $self->{prefixes};
+    my %yes   = map +($_ => 1), @{$yes || []};
+    my %no    = map +($_ => 1), @{$no  || []};
+    my $prefs = $self->{prefixes};
 
     $handler ||= 'SKIP_ALL';
+    weaken $self;
+
     bless
     sub { my ($doc, $values) = @_;
           my @res;
@@ -965,7 +983,8 @@ sub makeAnyElement
               }
               elsif(index($label, ':') >= 0)
               {   (my $prefix, $local) = split ':', $label, 2;
-                  $ns    = $self->namespaceForPrefix($prefix);
+                  my $match = first {$_->{prefix} eq $prefix} values %$prefs;
+                  my $ns    = $match ? $match->{uri} : undef;
                   $type  = pack_type $ns, $local;
               }
               else {next}  # not fully qualified, not an 'any'
@@ -974,7 +993,7 @@ sub makeAnyElement
               $no{$ns} and next if keys %no;
 
               my $value = delete $values->{$label} or next;
-              my ($attrs, $elems) = $self->_splitAnyList($path, $type, $value);
+              my ($attrs, $elems) = _split_any_list $path, $type, $value;
 
               $values->{$type} = $attrs if @$attrs;
               @$elems or next;
@@ -1045,7 +1064,7 @@ sub makeHook($$$$$$)
     error __x"writer only supports one production (replace) hook"
         if $replace && @$replace > 1;
 
-    return sub {()} if $replace && grep {$_ eq 'SKIP'} @$replace;
+    return sub {()} if $replace && grep $_ eq 'SKIP', @$replace;
 
     my @replace = $replace ? map $self->_decodeReplace($path,$_), @$replace :();
     my @before  = $before  ? map $self->_decodeBefore($path,$_),  @$before  :();
@@ -1101,22 +1120,23 @@ sub _decodeAfter($$)
 
 sub makeBlocked($$$)
 {   my ($self, $where, $class, $type) = @_;
+    my $err_type = $self->prefixed($type);
 
     # errors are produced in class=misfit to allow other choices to succeed.
       $class eq 'anyType'
     ? { st => sub { error __x"use of `{type}' blocked at {where}"
-              , type => $type, where => $where, _class => 'misfit';
+              , type => $err_type, where => $where, _class => 'misfit';
           }}
     : $class eq 'simpleType'
     ? { st => sub { error __x"use of {class} `{type}' blocked at {where}"
-              , class => $class, type => $type, where => $where
+              , class => $class, type => $err_type, where => $where
               , _class => 'misfit';
           }}
     : $class eq 'complexType'
     ? { elems => [] }
     : $class eq 'ref'
     ? { st => sub { error __x"use of referenced `{type}' blocked at {where}"
-              , type => $type, where => $where, _class => 'misfit';
+              , type => $err_type, where => $where, _class => 'misfit';
           }}
     : panic "blocking of $class for $type not implemented";
 }
